@@ -14,10 +14,12 @@ from datetime import datetime
 from datetime import timedelta
 from functools import singledispatch
 from pathlib import Path
+from typing import Any
 from typing import Dict
 from typing import Iterable
 from typing import List
 from typing import Optional
+from typing import Tuple
 from typing import Union
 
 import numpy as np
@@ -33,6 +35,10 @@ from rich.panel import Panel
 from rich.table import Table
 from rich.text import Text
 from toolz import itertoolz
+from toolz.dicttoolz import merge
+from transformers import AutoFeatureExtractor
+from transformers import AutoModelForImageClassification
+from transformers import pipeline
 
 from flyswot import core
 from flyswot import models
@@ -43,8 +49,8 @@ app = typer.Typer()
 
 
 @dataclass
-class ImagePredictionItem:
-    """Prediction for an image.
+class ImagePredictionArgmaxItem:
+    """Most confident predicted label for an item.
 
     Attributes:
         path: The Path to the image
@@ -89,7 +95,7 @@ class MultiLabelImagePredictionItem:
 class PredictionBatch:
     """Container for ImagePredictionItems"""
 
-    batch: List[ImagePredictionItem]
+    batch: List[ImagePredictionArgmaxItem]
 
     def __post_init__(self):
         """Returns a iterable of all predicted labels in batch"""
@@ -142,12 +148,13 @@ def try_predict_batch(batch, inference_session, bs):
         batch_predictions = inference_session.predict_batch(batch, bs)
         return batch_predictions, bad_batch
     except PIL.UnidentifiedImageError:
-        typer.echo("Corrupt imagess found in batch")
         bad_batch = True
         return batch, bad_batch
 
 
-def predict_files(files: List[Path], inference_session, bs, csv_fname) -> set:
+def predict_files(
+    files: List[Path], inference_session, bs, csv_fname
+) -> Tuple[set, int]:
     """Predict files"""
     with typer.progressbar(length=len(files)) as progress:
         images_checked = 0
@@ -165,13 +172,14 @@ def predict_files(files: List[Path], inference_session, bs, csv_fname) -> set:
             progress.update(len(batch))
             images_checked += len(batch)
         corrupt_images = set()
-        for batch in bad_batch_files:
-            for file in batch:
-                try:
-                    batch_predictions = inference_session.predict_batch(files, bs)
-                    write_batch_preds_to_csv(batch_predictions)
-                except PIL.UnidentifiedImageError:
-                    corrupt_images.add(file)
+        if bad_batch_files:
+            for batch in bad_batch_files:
+                for file in batch:
+                    try:
+                        batch_predictions = inference_session.predict_batch([file], bs)
+                        write_batch_preds_to_csv(batch_predictions, csv_fname)
+                    except PIL.UnidentifiedImageError:
+                        corrupt_images.add(file)
         return corrupt_images, images_checked
 
 
@@ -189,16 +197,17 @@ def predict_directory(
         resolve_path=True,
         help="Directory used to store the csv report",
     ),
-    pattern: str = typer.Option("fs", help="Pattern used to filter image filenames"),
-    bs: int = typer.Option(16, help="Batch Size"),
-    image_format: str = typer.Option(
-        ".tif",
-        help="Image format for flyswot to use for predictions, defaults to `*.tif`",
-    ),
-    model_name: str = typer.Option(
-        "latest", help="Which model flyswot should use for making predictions"
+    model_id: str = typer.Option(
+        "flyswot/convnext-tiny-224_flyswot",
+        help="The model flyswot should use for making predictions",
     ),
     model_path: str = None,
+    pattern: str = typer.Option("fs", help="Pattern used to filter image filenames"),
+    bs: int = typer.Option(16, help="Batch Size"),
+    image_formats: List[str] = typer.Option(
+        [".tif"],
+        help="Image format for flyswot to use for predictions, defaults to `*.tif`",
+    ),
 ):
     """Predicts against all images stored under DIRECTORY which match PATTERN in the filename.
 
@@ -207,21 +216,27 @@ def predict_directory(
     Creates a CSV report saved to `csv_save_dir`
     """
     start_time = time.perf_counter()
-    model_dir = models.ensure_model_dir()
-    model = models.ensure_model(model_dir)
-    onnxinference = OnnxInferenceSession(model.model, model.vocab)
-    files = sorted(core.get_image_files_from_pattern(directory, pattern, image_format))
+    # model_dir = models.ensure_model_dir()
+    # model = models.ensure_model(model_dir)
+    # onnxinference = OnnxInferenceSession(model.model, model.vocab)
+    huggingfaceinference = HuggingFaceInferenceSession(model=model_id)
+    files = sorted(
+        itertoolz.concat(
+            core.get_image_files_from_pattern(directory, pattern, image_format)
+            for image_format in image_formats
+        )
+    )
     check_files(files, pattern, directory)
     typer.echo(f"Found {len(files)} files matching {pattern} in {directory}")
     csv_fname = create_csv_fname(csv_save_dir)
     corrupt_images, images_checked = predict_files(
-        files, inference_session=onnxinference, bs=bs, csv_fname=csv_fname
+        files, inference_session=huggingfaceinference, bs=bs, csv_fname=csv_fname
     )
     if corrupt_images:
         print(corrupt_images)
     delta = timedelta(seconds=time.perf_counter() - start_time)
     print_inference_summary(
-        str(delta), pattern, directory, csv_fname, image_format, images_checked
+        str(delta), pattern, directory, csv_fname, image_formats, images_checked
     )
 
 
@@ -357,7 +372,7 @@ def create_csv_fname(csv_directory: Path) -> Path:
     """Creates a csv filename"""
     date_now = datetime.now()
     date_now = date_now.strftime("%Y_%m_%d_%H_%M")
-    fname = Path(date_now + ".csv")
+    fname = Path(f"{date_now}.csv")
     return Path(csv_directory / fname)
 
 
@@ -431,7 +446,7 @@ class InferenceSession(ABC):
     """Abstract class for inference sessions"""
 
     @abstractmethod
-    def __init__(self, model: Path, vocab: List):  # pragma: no cover
+    def __init__(self, model: Path, vocab: Optional[List] = None):  # pragma: no cover
         """Inference Sessions should init from a model file and vocab"""
         self.model = model
         self.vocab = vocab
@@ -442,9 +457,7 @@ class InferenceSession(ABC):
         pass
 
     @abstractmethod
-    def predict_batch(
-        self, model: Path, batch: Iterable[Path], bs: int
-    ):  # pragma: no cover
+    def predict_batch(self, batch: Iterable[Path], bs: int):  # pragma: no cover
         """Predict a batch"""
         pass
 
@@ -494,7 +507,7 @@ class OnnxInferenceSession(InferenceSession):  # pragma: no cover
 
     def predict_image(
         self, image: Path
-    ) -> Union[ImagePredictionItem, MultiLabelImagePredictionItem]:
+    ) -> Union[ImagePredictionArgmaxItem, MultiLabelImagePredictionItem]:
         """Predict a single image"""
         img = self._load_image(image)
         output_names = [o.name for o in self.session.get_outputs()]
@@ -502,9 +515,9 @@ class OnnxInferenceSession(InferenceSession):  # pragma: no cover
         if len(self.vocab) < 2:
             pred = self._postprocess(raw_result)
             arg_max = int(np.array(pred).argmax())
-            predicted_label = self.vocab_mappings[0][int(arg_max)]
+            predicted_label = self.vocab_mappings[0][arg_max]
             confidence = float(np.array(pred).max())
-            return ImagePredictionItem(image, predicted_label, confidence)
+            return ImagePredictionArgmaxItem(image, predicted_label, confidence)
         else:
             prediction_dicts = []
             for vocab_map, pred in zip(self.vocab_mappings, raw_result):
@@ -555,5 +568,48 @@ class OnnxInferenceSession(InferenceSession):  # pragma: no cover
             return MultiPredictionBatch(prediction_items)
 
 
+class HuggingFaceInferenceSession(InferenceSession):
+    "Huggingface inference session"
+
+    def __init__(self, model: str):
+        """Create Hugging Face Inference Session"""
+        self.model = AutoModelForImageClassification.from_pretrained(model)
+        self.feature_extractor = AutoFeatureExtractor.from_pretrained(model)
+        self.session = pipeline(
+            "image-classification",
+            model=self.model,
+            feature_extractor=self.feature_extractor,
+        )
+
+    def predict_image(self, image: Path):
+        """Predict single Image."""
+        return self.session(image, top_k=10)
+
+    def predict_batch(self, batch: Iterable[Path], bs: int):
+        """Predict batch of images"""
+        str_batch = [str(file) for file in batch]
+        predictions: List[List[Dict[Any]]] = self.session(
+            str_batch, batch_size=bs, top_k=20
+        )
+        prediction_dicts = [self._process_prediction_dict(pred) for pred in predictions]
+        all_pred = []
+        for file, pred in zip(str_batch, prediction_dicts):
+            # flysheet_other_pred = self._create_flysheet_other_predictions(pred)
+            item_pred = [pred]
+            prediction = MultiLabelImagePredictionItem(Path(file), item_pred)
+            all_pred.append(prediction)
+        return MultiPredictionBatch(all_pred)
+
+    def _process_prediction_dict(
+        self, prediction_item: List[Dict[str, float]]
+    ) -> Dict[float, str]:
+        return merge(
+            [
+                {prediction["score"]: prediction["label"]}
+                for prediction in prediction_item
+            ]
+        )
+
+
 if __name__ == "__main__":
     app()  # pragma: no cover
